<?php
namespace MyMenu\Controllers;

use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class Rate extends Simple
{

    protected $entityManager;
    protected $environment;
    protected $newCode;
    
    public function __construct($entityManager, $environment, $newCode)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->newCode = $newCode;        
    }
    public function rateSave(Request $request, Response $response)
    {
        $arr = $request->getRow();
        $checkIfAlreadyBookRegister = $this->checkIfThereIsAval($arr);

        if (count($checkIfAlreadyBookRegister) > 0) {
            $this->updateOldAval($checkIfAlreadyBookRegister[0]['NRAVALIACAOPRATO'], $arr);
        } else {
            $this->insertNewAval($arr);
        }
    }

    public function consSrvSave(Request $request, Response $response)
    {
        $srv = $request->getRow();
        if(!isset($srv['CDREGISTROPESSOA'])){
            $srv['CDREGISTROPESSOA'] = null;
        }
        if (isset($srv['isAnon']) && $srv['isAnon']) {
            if ($srv['IDCONSUMO'] === 'S') {
                $this->insertNewServCons($srv, $response);
            } else {
                $this->removeAnonServCons($srv);
            }
        } else {
            if ($srv['IDCONSUMO'] === 'S') {
                $this->verificaCpfCadastrado($srv);
                $this->reservaJaFeita($srv);
            }
            $this->loggedServConsSave($srv, $response);
        }
    }

    public function consSave(Request $request, Response $response)
    {
        $plates = $request->getRow();
        if(!empty($plates)) {
            if(!isset($plates[0]['CDREGISTROPESSOA'])){
                $plates[0]['CDREGISTROPESSOA'] = null;
            }
            $fazendoReserva = ($plates[0]['IDCONSUMO'] === 'S');
            $anonimo = false;
            if(isset($plates[0]['isAnon'])){
                if($plates[0]['isAnon'])
                    $anonimo = true;
            }
            if(!$anonimo && $fazendoReserva) {
                $this->verificaCpfCadastrado($plates[0]);
                $this->reservaJaFeita($plates[0], true);
            }
            foreach($plates as &$plate) {
                if(!isset($plate['CDREGISTROPESSOA'])){
                    $plate['CDREGISTROPESSOA'] = null;
                }                
                $NRAVALIACAOPRATO = $this->insertOrRemoveCons($plate);
                $plate['NRAVALIACAOPRATO'] = $NRAVALIACAOPRATO;
            }
        $response->addDataSet(new DataSet('data',array($plates)));
        }
        
    }
    
    public function insertOrRemoveCons($plate) {
        if (isset($plate['isAnon']) && $plate['isAnon']) {
            if ($plate['IDCONSUMO'] === 'S') {
                return $this->insertNewCons($plate);
            } else {
                $this->removeAnonCons($plate);
            }
        } else {
            return $this->loggedConsSave($plate);
        }
    }
    
    public function loggedServConsSave($srv, $response)
    {
        $checkIfAlreadyBookRegister = $this->checkIfThereIsSrvAval($srv);
        if (count($checkIfAlreadyBookRegister) > 0) {
            /*se estiver cancelando uma reseva precisa verificar o status para não remover pedido já retirado*/
            if(in_array($checkIfAlreadyBookRegister[0]['IDSTATUSRESERVA'], ['LIBERADO', 'RETIRADO']) && ($checkIfAlreadyBookRegister[0]['IDCONSUMO'] ==='S' && $srv['IDCONSUMO'] === 'N')){
                throw new \Exception("RESERVA_JA_RECEBIDA_USUARIO");
            }
            $this->updateOldSrvCons($checkIfAlreadyBookRegister[0]['NRAVALIACAOSERVICO'], $srv);
        } else {
            $this->insertNewServCons($srv, $response);
        }
    }
    
    public function loggedConsSave($plate)
    {
        $checkIfAlreadyBookRegister = $this->checkIfThereIsAval($plate);
        if (count($checkIfAlreadyBookRegister) > 0) {
            /*se estiver cancelando uma reseva precisa verificar o status para não remover pedido já retirado*/
            if(in_array($checkIfAlreadyBookRegister[0]['IDSTATUSRESERVA'], ['LIBERADO', 'RETIRADO']) && ($checkIfAlreadyBookRegister[0]['IDCONSUMO'] ==='S' && $plate['IDCONSUMO'] === 'N')){
                throw new \Exception("RESERVA_JA_RECEBIDA_USUARIO");
            }            
            $this->updateOldCons($checkIfAlreadyBookRegister[0]['NRAVALIACAOPRATO'], $plate);
        } else {
            return $this->insertNewCons($plate);
        }
    }

    public function checkIfThereIsSrvAval($srv)
    {
        $nrAvaliacaoServico = isset($srv['NRAVALIACAOSERVICO']) ? $srv['NRAVALIACAOSERVICO'] : null;
        $params = array(
            "CDFILIAL" => $srv['CDFILIAL'],
            "CDSERVFILI" => $srv['CDSERVFILI'],
            "CDDIETPAD" => $srv['CDDIETPAD'],
            "DTCARDAPIO" => $srv['DTCARDAPIO'],
            "NRUSUARIOAPP" => $srv['NRUSUARIOAPP'],
            "NRLOCALPESQ" => $srv['NRLOCALPESQ'],
            "NRORG" => $srv['NRORG'],
            "NRAVALIACAOSERVICO" => $nrAvaliacaoServico
        );

        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("SELECT * FROM AVALIACAOSERVICO WHERE (NRAVALIACAOSERVICO = :NRAVALIACAOSERVICO) OR (:NRAVALIACAOSERVICO IS NULL AND (CDDIETPAD =:CDDIETPAD OR :CDDIETPAD IS NULL) AND (CDFILIAL =:CDFILIAL OR :CDFILIAL IS NULL) AND (CDSERVFILI =:CDSERVFILI OR :CDSERVFILI IS NULL) AND (DTCARDAPIO =:DTCARDAPIO OR :DTCARDAPIO IS NULL) AND (NRUSUARIOAPP =:NRUSUARIOAPP OR :NRUSUARIOAPP IS NULL) AND (NRORG =:NRORG OR :NRORG IS NULL) AND (NRLOCALPESQ =:NRLOCALPESQ OR :NRLOCALPESQ IS NULL))");
        $statement->execute($params);
        return $statement->fetchAll();
    }
    
    public function checkIfThereIsAval($plate)
    {
        $nravaliacaoprato = isset($plate['NRAVALIACAOPRATO']) ? $plate['NRAVALIACAOPRATO'] : null;
        $params = array(
            "CDFILIAL" => $plate['CDFILIAL'],
            "CDSERVFILI" => $plate['CDSERVFILI'],
            "NRCARDAPIO" => $plate['NRCARDAPIO'],
            "CDDIETPAD" => $plate['CDDIETPAD'],
            "NRSEQPRATCAR" => $plate['NRSEQPRATCAR'],
            "CDFILAUXPRAT" => $plate['CDFILAUXPRAT'],
            "CDPRATO" => $plate['CDPRATO'],
            "DTCARDAPIO" => $plate['DTCARDAPIO'],
            "CDTIPOPRAT" => $plate['CDTIPOPRAT'],
            "NRUSUARIOAPP" => $plate['NRUSUARIOAPP'],
            "NRLOCALPESQ" => $plate['NRLOCALPESQ'],
            "NRORG" => $plate['NRORG'],
            "NRAVALIACAOPRATO" => $nravaliacaoprato
        );

        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("SELECT * FROM AVALIACAOPRATO WHERE (NRAVALIACAOPRATO = :NRAVALIACAOPRATO) OR (:NRAVALIACAOPRATO IS NULL AND (CDDIETPAD =:CDDIETPAD OR :CDDIETPAD IS NULL) AND (CDFILIAL =:CDFILIAL OR :CDFILIAL IS NULL) AND (CDSERVFILI =:CDSERVFILI OR :CDSERVFILI IS NULL) AND (NRCARDAPIO = :NRCARDAPIO OR :NRCARDAPIO IS NULL) AND (NRSEQPRATCAR =:NRSEQPRATCAR OR :NRSEQPRATCAR IS NULL) AND (CDFILAUXPRAT =:CDFILAUXPRAT OR :CDFILAUXPRAT IS NULL) AND (CDPRATO =:CDPRATO OR :CDPRATO IS NULL) AND (DTCARDAPIO =:DTCARDAPIO OR :DTCARDAPIO IS NULL) AND (CDTIPOPRAT =:CDTIPOPRAT OR :CDTIPOPRAT IS NULL) AND (NRUSUARIOAPP =:NRUSUARIOAPP OR :NRUSUARIOAPP IS NULL) AND (NRORG =:NRORG OR :NRORG IS NULL) AND (NRLOCALPESQ =:NRLOCALPESQ OR :NRLOCALPESQ IS NULL))");
        $statement->execute($params);
        return $statement->fetchAll();
    }

    public function insertNewServCons($srv, $response)
    {
        $nrAvaliacaoServico = intval($this->newCode->getCodeByTableName("AVALIACAOSERVICO", $this->environment->getNrorg()));
        $params = array(
            "NRAVALIACAOSERVICO" => $nrAvaliacaoServico,
            "IDCONSUMO" => $srv['IDCONSUMO'],
            "CDFILCENTPME" => $srv['CDFILCENTPME'],
            "CDTIPOCARD" => $srv['CDTIPOCARD'],
            "CDFILIAL" => $srv['CDFILIAL'],
            "CDSERVFILI" => $srv['CDSERVFILI'],
            "CDDIETPAD" => $srv['CDDIETPAD'],
            "DTCARDAPIO" => $srv['DTCARDAPIO'],
            "NRUSUARIOAPP" => $srv['NRUSUARIOAPP'],
            "NMUSUARIO" => $srv['NMUSUARIO'],
            "HRCONSUMO" => $srv['HRCONSUMO'],
            "DSCONSUMO" => $srv['DSCONSUMO'],
            "NRLOCALPESQ" => $srv['NRLOCALPESQ'],
            "CDREGISTROPESSOA" => $srv['CDREGISTROPESSOA'],
            "IDSTATUSRESERVA" => $srv['IDCONSUMO'] === 'S' ? 'AGENDADO' : null,
            "NRORG" => $srv['NRORG']
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("
            INSERT INTO AVALIACAOSERVICO
                (NRAVALIACAOSERVICO, IDCONSUMO, CDFILCENTPME, CDTIPOCARD,  CDFILIAL, CDSERVFILI, CDDIETPAD, DTCARDAPIO, NRUSUARIOAPP, NMUSUARIO, HRCONSUMO, DSCONSUMO, NRLOCALPESQ, CDREGISTROPESSOA, IDSTATUSRESERVA, NRORG)
            VALUES
                (:NRAVALIACAOSERVICO, :IDCONSUMO, :CDFILCENTPME, :CDTIPOCARD, :CDFILIAL, :CDSERVFILI, :CDDIETPAD, :DTCARDAPIO, :NRUSUARIOAPP, :NMUSUARIO, :HRCONSUMO, :DSCONSUMO, :NRLOCALPESQ, :CDREGISTROPESSOA, :IDSTATUSRESERVA, :NRORG)");
        $statement->execute($params);
        $response->addDataSet(new DataSet('data',array($nrAvaliacaoServico)));
    }

    public function insertNewCons($plate)
    {
        $nrAvaliacaoPrato = intval($this->newCode->getCodeByTableName("AVALIACAOPRATO", $this->environment->getNrorg()));
        $params = array(
            "NRAVALIACAOPRATO" => $nrAvaliacaoPrato,
            "CDFILCENTPME" => $plate['CDFILCENTPME'],
            "CDTIPOCARD" => $plate['CDTIPOCARD'],
            "NRCARDPME" => $plate['NRCARDPME'],
            "CDFILIAL" => $plate['CDFILIAL'],
            "CDSERVFILI" => $plate['CDSERVFILI'],
            "NRCARDAPIO" => $plate['NRCARDAPIO'],
            "CDDIETPAD" => $plate['CDDIETPAD'],
            "IDCONSUMO" => $plate['IDCONSUMO'],
            "NRSEQPRATCAR" => $plate['NRSEQPRATCAR'],
            "CDFILAUXPRAT" => $plate['CDFILAUXPRAT'],
            "CDPRATO" => $plate['CDPRATO'],
            "DTCARDAPIO" => $plate['DTCARDAPIO'],
            "CDTIPOPRAT" => $plate['CDTIPOPRAT'],
            "NRUSUARIOAPP" => $plate['NRUSUARIOAPP'],
            "NMUSUARIO" => $plate['NMUSUARIO'],
            "HRCONSUMO" => $plate['HRCONSUMO'],
            "DSCONSUMO" => $plate['DSCONSUMO'],
            "NRLOCALPESQ" => $plate['NRLOCALPESQ'],
            "CDREGISTROPESSOA" => $plate['CDREGISTROPESSOA'],
            "IDSTATUSRESERVA" => $plate['IDCONSUMO'] === 'S' ? 'AGENDADO' : null,
            "NRORG" => $plate['NRORG'],
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("
                INSERT INTO AVALIACAOPRATO
                    (NRAVALIACAOPRATO, CDFILCENTPME, CDTIPOCARD, NRCARDPME, CDFILIAL, CDSERVFILI, NRCARDAPIO, CDDIETPAD, IDCONSUMO, NRSEQPRATCAR, CDFILAUXPRAT, CDPRATO, CDTIPOPRAT,  NRUSUARIOAPP, NMUSUARIO, HRCONSUMO, DSCONSUMO, DTCARDAPIO, NRLOCALPESQ, CDREGISTROPESSOA, IDSTATUSRESERVA, NRORG)
                VALUES
                    (:NRAVALIACAOPRATO, :CDFILCENTPME, :CDTIPOCARD, :NRCARDPME, :CDFILIAL, :CDSERVFILI, :NRCARDAPIO, :CDDIETPAD, :IDCONSUMO, :NRSEQPRATCAR, :CDFILAUXPRAT, :CDPRATO, :CDTIPOPRAT, :NRUSUARIOAPP, :NMUSUARIO, :HRCONSUMO, :DSCONSUMO, :DTCARDAPIO, :NRLOCALPESQ, :CDREGISTROPESSOA, :IDSTATUSRESERVA, :NRORG)");
        $statement->execute($params);
        return $nrAvaliacaoPrato;
    }

    public function removeAnonServCons($srv)
    {
        $checkIfAlreadyBookRegister = $this->checkIfThereIsSrvAval($srv);
        if (count($checkIfAlreadyBookRegister) > 0) {
            /*se estiver cancelando uma reseva precisa verificar o status para não remover pedido já retirado*/
            if(in_array($checkIfAlreadyBookRegister[0]['IDSTATUSRESERVA'], ['LIBERADO', 'RETIRADO']) && ($checkIfAlreadyBookRegister[0]['IDCONSUMO'] ==='S' && $srv['IDCONSUMO'] === 'N')){
                throw new \Exception("RESERVA_JA_RECEBIDA_USUARIO");
            }
        }
        $SQL = "DELETE FROM AVALIACAOSERVICO WHERE ( CDFILCENTPME =:CDFILCENTPME OR :CDFILCENTPME IS NULL ) AND ( CDTIPOCARD =:CDTIPOCARD OR :CDTIPOCARD IS NULL ) AND ( CDDIETPAD =:CDDIETPAD OR :CDDIETPAD IS NULL ) AND ( CDFILIAL =:CDFILIAL OR :CDFILIAL IS NULL ) AND ( CDSERVFILI =:CDSERVFILI OR :CDSERVFILI IS NULL )  AND ( DTCARDAPIO =:DTCARDAPIO OR :DTCARDAPIO IS NULL ) AND ( NRUSUARIOAPP =:NRUSUARIOAPP OR :NRUSUARIOAPP IS NULL ) AND ( IDCONSUMO =:IDCONSUMO OR :IDCONSUMO IS NULL ) AND ( NRORG =:NRORG OR :NRORG IS NULL ) AND (NMUSUARIO = :NMUSUARIO) AND (HRCONSUMO = :HRCONSUMO) AND (NVL(NRLOCALPESQ, 0) = NVL(:NRLOCALPESQ, 0)) AND ROWNUM < 2";
        $params = array(
            "HRCONSUMO" => $srv['HRCONSUMO'],
            "NMUSUARIO" => $srv['NMUSUARIO'],
            "CDFILCENTPME" => $srv['CDFILCENTPME'],
            "CDTIPOCARD" => $srv['CDTIPOCARD'],
            "CDDIETPAD" => $srv['CDDIETPAD'],
            "CDFILIAL" => $srv['CDFILIAL'],
            "CDSERVFILI" => $srv['CDSERVFILI'],
            "DTCARDAPIO" => $srv['DTCARDAPIO'],
            "NRUSUARIOAPP" => $srv['NRUSUARIOAPP'],
            "NMUSUARIO" => $srv['NMUSUARIO'],
            "HRCONSUMO" => $srv['HRCONSUMO'],
            "NRLOCALPESQ" => $srv['NRLOCALPESQ'],
            "IDCONSUMO" => 'S',
            "NRORG" => $srv['NRORG'],
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare($SQL);
        $statement->execute($params);
    }
    
    public function removeAnonCons($plate)
    {
        $checkIfAlreadyBookRegister = $this->checkIfThereIsAval($plate);
        if (count($checkIfAlreadyBookRegister) > 0) {
            /*se estiver cancelando uma reseva precisa verificar o status para não remover pedido já retirado*/
            if(in_array($checkIfAlreadyBookRegister[0]['IDSTATUSRESERVA'], ['LIBERADO', 'RETIRADO']) && ($checkIfAlreadyBookRegister[0]['IDCONSUMO'] ==='S' && $plate['IDCONSUMO'] === 'N')){
                throw new \Exception("RESERVA_JA_RECEBIDA_USUARIO");
            }            
        }        
        $SQL = "DELETE FROM AVALIACAOPRATO WHERE ( CDFILCENTPME =:CDFILCENTPME OR :CDFILCENTPME IS NULL ) AND ( CDTIPOCARD =:CDTIPOCARD OR :CDTIPOCARD IS NULL ) AND ( NRCARDPME =:NRCARDPME OR :NRCARDPME IS NULL ) AND ( CDDIETPAD =:CDDIETPAD OR :CDDIETPAD IS NULL ) AND ( CDFILIAL =:CDFILIAL OR :CDFILIAL IS NULL ) AND ( CDSERVFILI =:CDSERVFILI OR :CDSERVFILI IS NULL ) AND ( NRCARDAPIO =:NRCARDAPIO OR :NRCARDAPIO IS NULL ) AND ( NRSEQPRATCAR =:NRSEQPRATCAR OR :NRSEQPRATCAR IS NULL ) AND ( CDFILAUXPRAT =:CDFILAUXPRAT OR :CDFILAUXPRAT IS NULL ) AND ( CDPRATO =:CDPRATO OR :CDPRATO IS NULL ) AND ( DTCARDAPIO =:DTCARDAPIO OR :DTCARDAPIO IS NULL ) AND ( CDTIPOPRAT =:CDTIPOPRAT OR :CDTIPOPRAT IS NULL ) AND ( NRUSUARIOAPP =:NRUSUARIOAPP OR :NRUSUARIOAPP IS NULL ) AND ( IDCONSUMO =:IDCONSUMO OR :IDCONSUMO IS NULL ) AND ( NRORG =:NRORG OR :NRORG IS NULL ) AND (NMUSUARIO = :NMUSUARIO) AND (HRCONSUMO = :HRCONSUMO) AND (NVL(NRLOCALPESQ, 0) = NVL(:NRLOCALPESQ, 0)) AND ROWNUM < 2";
        $params = array(
            "CDFILCENTPME" => $plate['CDFILCENTPME'],
            "CDTIPOCARD" => $plate['CDTIPOCARD'],
            "NRCARDPME" => $plate['NRCARDPME'],
            "CDDIETPAD" => $plate['CDDIETPAD'],
            "CDFILIAL" => $plate['CDFILIAL'],
            "CDSERVFILI" => $plate['CDSERVFILI'],
            "NRCARDAPIO" => $plate['NRCARDAPIO'],
            "NRSEQPRATCAR" => $plate['NRSEQPRATCAR'],
            "CDFILAUXPRAT" => $plate['CDFILAUXPRAT'],
            "CDPRATO" => $plate['CDPRATO'],
            "DTCARDAPIO" => $plate['DTCARDAPIO'],
            "CDTIPOPRAT" => $plate['CDTIPOPRAT'],
            "NRUSUARIOAPP" => $plate['NRUSUARIOAPP'],
            "NMUSUARIO" => $plate['NMUSUARIO'],
            "HRCONSUMO" => $plate['HRCONSUMO'],
            "NRLOCALPESQ" => $plate['NRLOCALPESQ'],
            "IDCONSUMO" => 'S',
            "NRORG" => $plate['NRORG'],
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare($SQL);
        $statement->execute($params);
    }

    public function updateOldSrvCons($oldReg, $srv)
    {
        $params = array(
            "IDCONSUMO" => $srv['IDCONSUMO'],
            "NRAVALIACAOSERVICO" => $oldReg,
            "HRCONSUMO" => isset($srv['HRCONSUMO'])? $srv['HRCONSUMO'] : null,
            "DSCONSUMO" => isset($srv['DSCONSUMO'])? $srv['DSCONSUMO'] : null,
            "CDREGISTROPESSOA" => isset($srv['CDREGISTROPESSOA'])? $srv['CDREGISTROPESSOA'] : null,
            "IDSTATUSRESERVA" => $srv['IDCONSUMO'] === 'S' ? 'AGENDADO' : null
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("UPDATE AVALIACAOSERVICO SET IDCONSUMO = :IDCONSUMO, HRCONSUMO = :HRCONSUMO, DSCONSUMO = :DSCONSUMO, CDREGISTROPESSOA = :CDREGISTROPESSOA,  IDSTATUSRESERVA = :IDSTATUSRESERVA WHERE NRAVALIACAOSERVICO = :NRAVALIACAOSERVICO");
        $statement->execute($params);
    }

    public function updateOldCons($oldReg, $plate)
    {
        
        $HRCONSUMO = isset($plate['HRCONSUMO']) ? $plate['HRCONSUMO'] : null;
        $DSCONSUMO = isset($plate['DSCONSUMO']) ? $plate['DSCONSUMO'] : null;
        $params = array(
            "IDCONSUMO" => $plate['IDCONSUMO'],
            "NRAVALIACAOPRATO" => $oldReg,
            "HRCONSUMO" => $HRCONSUMO,
            "DSCONSUMO" => $DSCONSUMO,
            "CDREGISTROPESSOA" => isset($plate['CDREGISTROPESSOA'])? $plate['CDREGISTROPESSOA'] : null,
            "IDSTATUSRESERVA" => $plate['IDCONSUMO'] === 'S' ? 'AGENDADO' : null            
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("UPDATE AVALIACAOPRATO SET IDCONSUMO = :IDCONSUMO, HRCONSUMO = :HRCONSUMO, DSCONSUMO = :DSCONSUMO, CDREGISTROPESSOA = :CDREGISTROPESSOA,  IDSTATUSRESERVA = :IDSTATUSRESERVA WHERE NRAVALIACAOPRATO = :NRAVALIACAOPRATO");
        $statement->execute($params);
    }

    public function insertNewAval($plate)
    {
        $params = array(
            "NRORGNOVOCODIGO" => $plate['NRORG'],
            "CDFILCENTPME" => $plate['CDFILCENTPME'],
            "CDTIPOCARD" => $plate['CDTIPOCARD'],
            "NRCARDPME" => $plate['NRCARDPME'],
            "CDFILIAL" => $plate['CDFILIAL'],
            "CDSERVFILI" => $plate['CDSERVFILI'],
            "NRCARDAPIO" => $plate['NRCARDAPIO'],
            "CDDIETPAD" => $plate['CDDIETPAD'],
            "NRSEQPRATCAR" => $plate['NRSEQPRATCAR'],
            "CDFILAUXPRAT" => $plate['CDFILAUXPRAT'],
            "CDPRATO" => $plate['CDPRATO'],
            "DTCARDAPIO" => $plate['DTCARDAPIO'],
            "CDTIPOPRAT" => $plate['CDTIPOPRAT'],
            "VRAVALIACAOPRATO" => $plate['VRAVALIACAOPRATO'],
            "DSAVALIACAOPRATO" => isset($plate['DSAVALIACAOPRATO']) ? $plate['DSAVALIACAOPRATO'] : null,
            "NRUSUARIOAPP" => $plate['NRUSUARIOAPP'],
            "NRLOCALPESQ" => $plate['NRLOCALPESQ'],
            "NRORG" => $plate['NRORG'],
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("
                DECLARE V_SEQUENCIAL VARCHAR2(20);
                BEGIN NOVO_CODIGO('AVALIACAOPRATO', V_SEQUENCIAL,1,:NRORGNOVOCODIGO);
                INSERT INTO AVALIACAOPRATO
                    (NRAVALIACAOPRATO, CDFILCENTPME, CDTIPOCARD, NRCARDPME, CDFILIAL, CDSERVFILI, NRCARDAPIO, CDDIETPAD, NRSEQPRATCAR, CDFILAUXPRAT, CDPRATO, CDTIPOPRAT, VRAVALIACAOPRATO, DSAVALIACAOPRATO, NRUSUARIOAPP, DTCARDAPIO, NRLOCALPESQ, NRORG)
                VALUES
                    (TO_NUMBER(V_SEQUENCIAL), :CDFILCENTPME, :CDTIPOCARD, :NRCARDPME, :CDFILIAL, :CDSERVFILI, :NRCARDAPIO, :CDDIETPAD, :NRSEQPRATCAR, :CDFILAUXPRAT, :CDPRATO, :CDTIPOPRAT, :VRAVALIACAOPRATO, :DSAVALIACAOPRATO, :NRUSUARIOAPP, :DTCARDAPIO, :NRLOCALPESQ, :NRORG);
                END;
                ");
        $statement->execute($params);
    }

    public function updateOldAval($oldReg, $plate)
    {
        $params = array(
            "VRAVALIACAOPRATO" => $plate['VRAVALIACAOPRATO'],
            "DSAVALIACAOPRATO" => isset($plate['DSAVALIACAOPRATO']) ? $plate['DSAVALIACAOPRATO'] : null,
            "NRAVALIACAOPRATO" => $oldReg,
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare("UPDATE AVALIACAOPRATO SET VRAVALIACAOPRATO = :VRAVALIACAOPRATO, DSAVALIACAOPRATO = :DSAVALIACAOPRATO WHERE NRAVALIACAOPRATO = :NRAVALIACAOPRATO");
        $statement->execute($params);
    }
    
    public function reservaJaFeita($registro, $porPrato=false){
        $SQL = "SELECT 1 AS EXISTE FROM AVALIACAOSERVICO WHERE ( CDFILCENTPME = :CDFILCENTPME OR :CDFILCENTPME IS NULL ) AND ( CDTIPOCARD = :CDTIPOCARD OR :CDTIPOCARD IS NULL ) AND ( CDDIETPAD = :CDDIETPAD OR :CDDIETPAD IS NULL ) AND ( CDFILIAL = :CDFILIAL OR :CDFILIAL IS NULL ) AND ( CDSERVFILI = :CDSERVFILI OR :CDSERVFILI IS NULL ) AND ( DTCARDAPIO = :DTCARDAPIO OR :DTCARDAPIO IS NULL ) AND ( NRUSUARIOAPP = :NRUSUARIOAPP OR :NRUSUARIOAPP IS NULL ) AND ( NRLOCALPESQ = :NRLOCALPESQ OR :NRLOCALPESQ IS NULL ) AND IDCONSUMO = 'S'";
        if($porPrato)
            $SQL = "SELECT 1 AS EXISTE FROM AVALIACAOPRATO WHERE ( CDFILCENTPME = :CDFILCENTPME OR :CDFILCENTPME IS NULL ) AND ( CDTIPOCARD = :CDTIPOCARD OR :CDTIPOCARD IS NULL ) AND ( CDDIETPAD = :CDDIETPAD OR :CDDIETPAD IS NULL ) AND ( CDFILIAL = :CDFILIAL OR :CDFILIAL IS NULL ) AND ( CDSERVFILI = :CDSERVFILI OR :CDSERVFILI IS NULL ) AND ( DTCARDAPIO = :DTCARDAPIO OR :DTCARDAPIO IS NULL ) AND ( NRUSUARIOAPP = :NRUSUARIOAPP OR :NRUSUARIOAPP IS NULL ) AND ( NRLOCALPESQ = :NRLOCALPESQ OR :NRLOCALPESQ IS NULL ) AND IDCONSUMO = 'S' AND CDPRATO IS NOT NULL";
        $params = array(
            "CDFILCENTPME" => $registro['CDFILCENTPME'],
            "CDTIPOCARD" => $registro['CDTIPOCARD'],
            "CDDIETPAD" => $registro['CDDIETPAD'],
            "CDFILIAL" => $registro['CDFILIAL'],
            "CDSERVFILI" => $registro['CDSERVFILI'],
            "DTCARDAPIO" => $registro['DTCARDAPIO'],
            "NRUSUARIOAPP" => $registro['NRUSUARIOAPP'],
            "NRLOCALPESQ" => $registro['NRLOCALPESQ']
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare($SQL);
        $statement->execute($params);
        $resultado = $statement->fetchAll();
        if(count($resultado) > 0){
            throw new \Exception("RESERVA_JA_REALIZADA_USUARIO");
        }
    }
    
    public function verificaCpfCadastrado($registro) {
        $SQL = "SELECT NRUSUARIOAPP FROM USUARIOAPP WHERE CDREGISTROPESSOA = :CDREGISTROPESSOA";
        $params = array(
            "CDREGISTROPESSOA" => $registro['CDREGISTROPESSOA']
        );
        $conn = $this->entityManager->getConnection();
        $statement = $conn->prepare($SQL);
        $statement->execute($params);
        $resultado = $statement->fetchAll();
        if(count($resultado) > 0){
            foreach ($resultado as $item) {
                if ($item['NRUSUARIOAPP'] != $registro['NRUSUARIOAPP']) {
                    throw new \Exception("CPF_JA_CADASTRADO_USUARIO");
                }
            }
        }else{
            $params = array(
                "CDREGISTROPESSOA" => $registro['CDREGISTROPESSOA'],
                "NRUSUARIOAPP" => $registro['NRUSUARIOAPP']
            );            
            $conn = $this->entityManager->getConnection();
            $statement = $conn->prepare("UPDATE USUARIOAPP SET CDREGISTROPESSOA = :CDREGISTROPESSOA WHERE NRUSUARIOAPP = :NRUSUARIOAPP");
            $statement->execute($params);  
        }
    }
}
