function LogDecorator(ZhuLogHelper) {
    /* Widget Getters */
    var getContentWidget = function(gridWidget) {
        return gridWidget.widgets[0].widgets[0];
    };

    /* Field Names */
    const ROUTE_FIELD_NAME              = 'ROUTE';
    const ROUTE_LABEL_FIELD_NAME        = 'ROUTE_LABEL';
    const TIME_FIELD_NAME               = 'TIMESTAMP';
    const USER_ID_FIELD_NAME            = 'USER_ID';
    const USER_NAME_FIELD_NAME          = 'USER_NAME';
    const CONTAINER_LABEL_FIELD_NAME    = 'CONTAINER_LABEL';
    const WIDGET_LABEL_FIELD_NAME       = 'WIDGET_LABEL';
    const CONTENT_FIELD_NAME            = 'CONTENT';
    const CONTEXT_DATA_FIELD_NAME       = 'CONTEXT_DATA';
    const USER_DATA_FIELD_NAME          = 'USER_DATA';
    const HTTP_RESPONSE_CODE_FIELD_NAME = 'HTTP_RESPONSE_CODE';
    const REQUEST_TYPE_FIELD_NAME       = 'REQUEST_TYPE';
    const REQUEST_ID_FIELD_NAME         = 'REQ_ID';
    const CONTENT_FILTER_FIELD_NAME     = 'contentFilter';
    const TIME_START_FIELD_NAME         = 'TIMESTAMP_START';
    const TIME_END_FIELD_NAME           = 'TIMESTAMP_END';
    const TYPE_FIELD_NAME               = 'TYPE';

    /* Field Labels */
    const USER_ID_FIELD_LABEL         = 'User Id';
    const USER_NAME_FIELD_LABEL       = 'User Name';
    const CONTAINER_LABEL_FIELD_LABEL = 'Screen';
    const WIDGET_LABEL_FIELD_LABEL    = 'Section Screen';
    const ROUTE_LABEL_FIELD_LABEL     = 'Action';

    /* Field Templates */
    const TEXT_EDIT_TEMPLATE = 'field/text-edit.html';
    const SELECT_NATIVE_TEMPLATE = 'field/select-native.html';
    const SELECT_MULTIPLE_TEMPLATE = 'field/select-multiple.html';

    var filterFieldsInAdvancedFieldGroup = [REQUEST_ID_FIELD_NAME, TYPE_FIELD_NAME, HTTP_RESPONSE_CODE_FIELD_NAME, CONTENT_FIELD_NAME];
    var fieldsNotUpdateTemplate = [ROUTE_LABEL_FIELD_NAME];

    var updateWidgetFields = function (widget) {
        var routeField          = ZhuLogHelper.getField(widget, ROUTE_FIELD_NAME);
        var userIdField         = Util.clone(routeField);
        var userNameField       = Util.clone(routeField);
        var containerLabelField = Util.clone(routeField);
        var widgetLabelField    = Util.clone(routeField);
        var routeLabelField     = Util.clone(routeField);

        userIdField.name  = USER_ID_FIELD_NAME;
        userIdField.label = USER_ID_FIELD_LABEL;

        userNameField.name  = USER_NAME_FIELD_NAME;
        userNameField.label = USER_NAME_FIELD_LABEL;

        containerLabelField.name  = CONTAINER_LABEL_FIELD_NAME;
        containerLabelField.label = CONTAINER_LABEL_FIELD_LABEL;

        widgetLabelField.name  = WIDGET_LABEL_FIELD_NAME;
        widgetLabelField.label = WIDGET_LABEL_FIELD_LABEL;

        routeLabelField.name  = ROUTE_LABEL_FIELD_NAME;
        routeLabelField.label = ROUTE_LABEL_FIELD_LABEL;

        ZhuLogHelper.addField (widget, userIdField,         1);
        ZhuLogHelper.addField (widget, userNameField,       2);
        ZhuLogHelper.moveField(widget, TIME_FIELD_NAME,     3);
        ZhuLogHelper.addField (widget, containerLabelField, 4);
        ZhuLogHelper.addField (widget, widgetLabelField,    5);
        ZhuLogHelper.addField (widget, routeLabelField,     6);
        ZhuLogHelper.moveField(widget, ROUTE_FIELD_NAME,    7);

        ZhuLogHelper.editField(widget, ROUTE_LABEL_FIELD_NAME, {
            valueField: ROUTE_FIELD_NAME,
            descriptionField: ROUTE_LABEL_FIELD_NAME,
            template: SELECT_NATIVE_TEMPLATE,
            maxWidth: '500px',
            dataSource: {
                data: window.routesTranslate || []
            },
            filterProperties: {
                template: SELECT_MULTIPLE_TEMPLATE,
                operator: 'IN'
            }
        });
    };

    var updateWidgetFieldsGridOnly = function(gridWidget) {
        gridWidget.fields.forEach(function(field) {
            if(field.name && !(filterFieldsInAdvancedFieldGroup.indexOf(field.name) != -1)) {
                field.fieldGroup = null;
            }

            if(field.filterProperties) {
                if(fieldsNotUpdateTemplate.indexOf(field.name) == -1) {
                    field.template = field.filterProperties.template;
                }
                if(field.filterProperties.template == TEXT_EDIT_TEMPLATE) {
                    field.filterProperties.operator = 'LIKE';
                }
            }
        });

        //General filter fields
        ZhuLogHelper.removeField(gridWidget, CONTENT_FILTER_FIELD_NAME);
        ZhuLogHelper.editField(gridWidget, USER_NAME_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, TIME_START_FIELD_NAME, {
            'class': 3,
            'datetime-lang': 'pt-BR',
            'datetime-dateFormat': 'dd/mm/yy',
            'datetime-timeFormat': 'hh:ii'
        });
        ZhuLogHelper.editField(gridWidget, TIME_END_FIELD_NAME, {
            'class': 3,
            'datetime-lang': 'pt-BR',
            'datetime-dateFormat': 'dd/mm/yy',
            'datetime-timeFormat': 'hh:ii'
        });
        ZhuLogHelper.editField(gridWidget, CONTAINER_LABEL_FIELD_NAME, {class: 3});
        ZhuLogHelper.editField(gridWidget, WIDGET_LABEL_FIELD_NAME, {class: 3});
        ZhuLogHelper.editField(gridWidget, ROUTE_LABEL_FIELD_NAME, {class: 3});
        ZhuLogHelper.editField(gridWidget, ROUTE_FIELD_NAME, {class: 3});

        //Advanced filter fields
        ZhuLogHelper.editField(gridWidget, REQUEST_ID_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, TYPE_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, CONTENT_FIELD_NAME, {
            isVisible: false,
            fieldGroup: 'advancedFilter',
            template : TEXT_EDIT_TEMPLATE,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE',
                class    : 12
            }
        });

        ZhuLogHelper.moveField(gridWidget, TIME_START_FIELD_NAME, 3);
        ZhuLogHelper.moveField(gridWidget, TIME_END_FIELD_NAME,   4);
    };

    var updateWidgetFieldsFormOnly = function(formWidget) {
        ZhuLogHelper.removeField(formWidget, CONTEXT_DATA_FIELD_NAME);
        ZhuLogHelper.removeField(formWidget, USER_DATA_FIELD_NAME);

        ZhuLogHelper.editField(formWidget, USER_NAME_FIELD_NAME,          {class: 6});
        ZhuLogHelper.editField(formWidget, TIME_FIELD_NAME,               {class: 4});
        ZhuLogHelper.editField(formWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {class: 2});
        ZhuLogHelper.editField(formWidget, REQUEST_TYPE_FIELD_NAME,       {class: 4});
        ZhuLogHelper.editField(formWidget, WIDGET_LABEL_FIELD_NAME,       {class: 6});
        ZhuLogHelper.editField(formWidget, CONTAINER_LABEL_FIELD_NAME,    {class: 6});
        ZhuLogHelper.editField(formWidget, ROUTE_FIELD_NAME,              {class: 6});
        ZhuLogHelper.editField(formWidget, ROUTE_LABEL_FIELD_NAME,        {class: 6});
    };

    var convertDateValue = function(value) {
        return "TO_DATE('" + value + "', 'DD/MM/YYYY HH24:MI')";
    };

    this.decorateWidget = function(gridWidget) {
        updateWidgetFields(gridWidget);
        updateWidgetFieldsGridOnly(gridWidget);

        var formWidget = getContentWidget(gridWidget);
        updateWidgetFields(formWidget);
        updateWidgetFieldsFormOnly(formWidget);

        gridWidget.currentRowDefaultFilter = {
            TIMESTAMP_START: moment().add(-1, 'days').format('DD/MM/YYYY') + ' 00:00',
            TIMESTAMP_END  : moment().format('DD/MM/YYYY') + ' 23:59'
        };
    };

    this.normalizeRows = function(rows) {
        rows = rows.map(function(row) {
            var userData  = JSON.parse(row.USER_DATA);
            row.USER_ID   = userData.CDOPERADOR;
            row.USER_NAME = userData.NMOPERADOR;

            var contextData     = JSON.parse(row.CONTEXT_DATA);
            row.CONTAINER_LABEL = contextData.containerLabel;
            row.WIDGET_LABEL    = contextData.widgetLabel;
            row.ROUTE_LABEL     = row.ROUTE;

            return row;
        });
        return rows;
    };

    this.normalizeFilters = function(filters) {
        filters = filters.map(function(filter) {
            if(filter.name == TIME_START_FIELD_NAME || filter.name == TIME_END_FIELD_NAME) {
                filter.value = convertDateValue(filter.value);
            }
            return filter;
        });
        return filters;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LogDecorator', LogDecorator);
});
